  -----------------------------
  -- INIT
  -----------------------------
  
  local addonName = ...
  local addonTitle = select(2, GetAddOnInfo(addonName))

  LortiUIConfig = LibStub("AceAddon-3.0"):NewAddon(addonName, "AceConsole-3.0")

  local options = {
      name = addonName,
      handler = LortiUIConfig,
      type = "group",
      args = {
        artframe = {
          type = "select",
          name = NORMAL_FONT_COLOR_CODE .. "Artframe Status" .. FONT_COLOR_CODE_CLOSE,
          desc = "Show or Hide the Gryphons next to the Main Actionbar",
          style = "radio",
          values = { [1] = "Hide", [2] = "Gryphon", [3] = "Lion"},
          get = "GetArtFrame",
          set = "SetArtFrame",
          descStyle = "inline",
          width = "full",
          order = 0,
        },
        hotkey = {
          type = "toggle",
          name = NORMAL_FONT_COLOR_CODE .. "Show Hotkey Text" .. FONT_COLOR_CODE_CLOSE,
          desc = "Show or Hide the Hotkey Text in Actionbars",
          get = "Getstatushotkeytext",
          set = "UpdateHotkeytext",
          descStyle = "inline",
          width = "full",
          order = 1,
        },
        macro = {
          type = "toggle",
          name = NORMAL_FONT_COLOR_CODE .. "Show Macro Name" .. FONT_COLOR_CODE_CLOSE,
          desc = "Show or Hide the Macro Name in Actionbars",
          get = "Getstatusmarcotext",
          set = "UpdateMacrotext",
          descStyle = "inline",
          width = "full",
          order = 2,
        },
      }
  }

  local optionsTable = LibStub("AceConfig-3.0"):RegisterOptionsTable(addonName, options)
  local AceConfigDialog = LibStub("AceConfigDialog-3.0")

  local defaults = {
    profile =  {
      artframe = 1,
      hotkey = true,
      macro = true,
    },
  }
  
  function LortiUIConfig:OnInitialize()
      self.db = LibStub("AceDB-3.0"):New(addonName, defaults, true)
      AceConfigDialog:AddToBlizOptions(addonName, addonTitle)
  end

  function LortiUIConfig:OnEnable()
    self:UpdateInterface()
  end
  
  function LortiUIConfig:OnDisable()
    self:UpdateInterface()
  end

  function LortiUIConfig:UpdateInterface()
    self:SetArtFrame()
    self:UpdateHotkeytext()
    self:UpdateMacrotext()
  end

  function LortiUIConfig:GetArtFrame(info)
    return LortiUIConfig.db.profile.artframe
  end 

  -- ArtFrame
  function LortiUIConfig:SetArtFrame(info, val)
    local statusdb = LortiUIConfig.db.profile.artframe
    if val then
      LortiUIConfig.db.profile.artframe = val
      statusdb = val
    end

    if statusdb == 1 then -- Hide
      MainMenuBarArtFrame.LeftEndCap:Hide()
      MainMenuBarArtFrame.RightEndCap:Hide()
    elseif statusdb == 2 then -- Gryphon
      MainMenuBarArtFrame.LeftEndCap:Show()
      MainMenuBarArtFrame.RightEndCap:Show()
      MainMenuBarArtFrame.LeftEndCap:SetTexture("Interface\\MAINMENUBAR\\UI-MainMenuBar-EndCap-Dwarf.blp")
      MainMenuBarArtFrame.RightEndCap:SetTexture("Interface\\MAINMENUBAR\\UI-MainMenuBar-EndCap-Dwarf.blp")
      MainMenuBarArtFrame.LeftEndCap:SetSize(128, 86)
      MainMenuBarArtFrame.RightEndCap:SetSize(128, 86)
      MainMenuBarArtFrame.LeftEndCap:SetTexCoord(0/128, 128/128, 42/128, 128/128)
      MainMenuBarArtFrame.RightEndCap:SetTexCoord(128/128, 0/128, 42/128, 128/128)
    else -- Lion
      MainMenuBarArtFrame.LeftEndCap:Show()
      MainMenuBarArtFrame.RightEndCap:Show()
      MainMenuBarArtFrame.LeftEndCap:SetTexture("Interface\\MAINMENUBAR\\UI-MainMenuBar-EndCap-Human.blp")
      MainMenuBarArtFrame.RightEndCap:SetTexture("Interface\\MAINMENUBAR\\UI-MainMenuBar-EndCap-Human.blp")
      MainMenuBarArtFrame.LeftEndCap:SetSize(128, 86)
      MainMenuBarArtFrame.RightEndCap:SetSize(128, 86)
      MainMenuBarArtFrame.LeftEndCap:SetTexCoord(0/128, 128/128, 42/128, 128/128)
      MainMenuBarArtFrame.RightEndCap:SetTexCoord(128/128, 0/128, 42/128, 128/128)
    end
  end

  -- Hotkey Text
  function LortiUIConfig:Getstatushotkeytext(info)
    return LortiUIConfig.db.profile.hotkey
  end 

  function LortiUIConfig:UpdateHotkeytext(info)
    if (LortiUIConfig.db.profile.hotkey == nil) then return end
    local _G = getfenv(0)
    local buttons = {
      "ActionButton",
      "MultiBarBottomLeftButton",
      "MultiBarBottomRightButton",
      "MultiBarLeftButton",
      "MultiBarRightButton",
      "PetActionButton"
    }

    if LortiUIConfig.db.profile.hotkey == true then
      for i = 1, getn(buttons) do
        for n = 1, 12 do
          local o = _G[buttons[i]..n.."HotKey"]
          if (o) then
            o:Hide()
          end
        end
      end
      LortiUIConfig.db.profile.hotkey = false
    else
      for i = 1, getn(buttons) do
        for n = 1, 12 do
          local o = _G[buttons[i]..n.."HotKey"]
          if (o) then
            o:Show()
          end
        end
      end
      LortiUIConfig.db.profile.hotkey = true
    end
  end

  -- Macro Text
  function LortiUIConfig:Getstatusmarcotext()
    return LortiUIConfig.db.profile.macro
  end 

  function LortiUIConfig:UpdateMacrotext(info)
    if (LortiUIConfig.db.profile.macro == nil) then return end
    local _G = getfenv(0)
    local buttons = {
      "ActionButton",
      "MultiBarBottomLeftButton",
      "MultiBarBottomRightButton",
      "MultiBarLeftButton",
      "MultiBarRightButton",
    }
    
    if LortiUIConfig.db.profile.macro == true then
      for i = 1, getn(buttons) do
        for n = 1, NUM_ACTIONBAR_BUTTONS do
          _G[buttons[i]..n.."Name"]:Hide()
        end
      end
      LortiUIConfig.db.profile.macro = false
    else
      for i = 1, getn(buttons) do
        for n = 1, NUM_ACTIONBAR_BUTTONS do
          _G[buttons[i]..n.."Name"]:Show()
        end
      end
      LortiUIConfig.db.profile.macro = true
    end
  end